<?php

/**
 * Access the Hackpad REST API.
 */
class HackpadApi {
  var $base_uri;
  var $client_id;
  var $secret;

  /**
   * Static function to use to create a new Hackpad connection or return an
   * existing connection. Calling methods should almost always use this to
   * return a connection to the API.
   */
  public static function api() {
    
    static $hackpadApi;
    
    if ($hackpadApi === NULL) {
      $hackpadApi = new HackpadApi();
    }

    return $hackpadApi;
  }

  /**
   * Construct a new API connection.
   */
  public function __construct() {
    $subdomain = variable_get('hackpad_subdomain', FALSE);
    $this->base_uri = "https://$subdomain.hackpad.com/api/1.0";

    $this->client_id = variable_get('hackpad_client_id', FALSE);
    $this->secret = variable_get('hackpad_secret', FALSE);
  }

  /**
   * Make a request to the Hackpad API.
   *
   * @param $resource
   *   The resource within the Hackpad API to access, such as 'pad/create'.
   * @param $options
   *   An array of options to pass to drupal_http_request(). Can also contain a
   *   'query' key containing an array of query parameters.
   *
   * @throws HackpadRequestException
   *
   * @see drupal_http_request()
   *
   * @return
   *   An object decoded from the response data.
   */
  protected function request($resource, $options = array()) {
    $uri = $this->base_uri . "/$resource";
    $method = isset($options['method']) ? $options['method'] : 'GET';
    
    $query = array();
    if (isset($options['query'])) {
      $query = $options['query'];
    }
    if(!is_array($options['headers'])){
      $options['headers'] = array();
    }

    $signed_uri = url($uri, array('query' => $this->signRequest($uri, $method, $query)));
    $response = drupal_http_request($signed_uri, $options['headers'], $method, '');
//dpm(array('Request:','signed_url' => $signed_uri, 'options' => $options, 'response' => $response, 'method' => $method, 'query' => $query));
    if ($response->code != 200) {
//      throw new HackpadRequestException($response->error, $response->code);
    }

    return $response;
  }

  /**
   * Helper function to return JSON-decoded data.
   */
  protected function request_json($resource, $options = array()) {
    return json_decode($this->request($resource, $options)->data);
  }

  /**
   * Sign a request to the Hackpad API using oauth 1.0. The Hackpad API doesn't
   * use tokens or anything beyond a very basic oauth request.
   *
   * @params $uri
   *   The URI to make the request to.
   * @param $method
   *   The HTTP method used for the request.
   * @param $query
   *   An optional array of query parameters to add to the URI.
   *
   * @return
   *   An array of URL parameters suitable for appending to a request made to
   *   $uri.
   */
  protected function signRequest($uri, $method, $query = array()) {
    $oauth = $query;
    $oauth['oauth_consumer_key'] = $this->client_id;
    $oauth['oauth_nonce'] = $this->nonce();
    $oauth['oauth_signature_method'] = 'HMAC-SHA1';
    $oauth['oauth_timestamp'] = time();
    $oauth['oauth_version'] = '1.0';

    // All parameters must be in alphabetical order for oauth hashing.
    ksort($oauth);

    // The most complicated part of the request - generating the signature.
    // The string to sign contains the HTTP method, the URL we are loading,
    // and all of our query parameters each URL encoded. Then, we concatenate
    // them with ampersands into a single string to hash.
    $http_verb = urlencode($method);
    $resource_url = urlencode($uri);
    $url_parameters = urlencode($this->buildQuery($oauth));
    $sig_string = $http_verb . '&' . $resource_url . '&' . $url_parameters;

    // Since we only have one oauth token (our shared secret) we only have to
    // use it as our hmac key. However, we still have to append an & to it as
    // if we were using it with additional tokens.
    $secret = urlencode($this->secret) . '&';

    // This is a hash of the consumer key and the "base string". Note that we
    // have to get the raw_output from hash_hmac but then base64 encode the
    // binary data result.
    $oauth['oauth_signature'] = base64_encode(hash_hmac('sha1', $sig_string, $secret, TRUE));
//dpm(array('secret' => $secret, 'oauth' => $oauth, 'seed' => array('sha1', $sig_string, $secret, TRUE)));
    return $oauth;
  }

  /**
   * A replication of drupal_http_build_query(), which is not available in D6
   * @param $query
   *  An associative array keyed by name => value for request parameters
   * @param $parent
   *  From Drupal API: "Internal use only. Used to build the $query array key for nested items."
   * @return "A rawurlencoded string which can be used as or appended to the URL query string."
   */
  protected function buildQuery($query, $parent = ''){
    $params = array();

    foreach ($query as $key => $value) {
      $key = ($parent ? $parent . '[' . rawurlencode($key) . ']' : rawurlencode($key));
  
      // Recurse into children.
      if (is_array($value)) {
        $params[] = $this->buildQuery($value, $key);
      }
      // If a query parameter value is NULL, only append its key.
      elseif (!isset($value)) {
        $params[] = $key;
      }
      else {
        // For better readability of paths in query strings, we decode slashes.
        $params[] = $key . '=' . str_replace('%2F', '/', rawurlencode($value));
      }
    }
  
    return implode('&', $params);
  }

  /**
    * Generate a unique string for use with oauth requests.
    *
    * @return
    *   A unique string suitable to use with an oauth nonce parameter.
    */
  protected static function nonce() {
    $mt = microtime();
    $rand = mt_rand();

    return md5($mt . $rand); // md5s look nicer than numbers
  }

  /**
   * Create a new HackPad with the given type and body.
   *
   * @param $content_type
   *   A MIME content type indicating the type of $body. One of text/html,
   *   text/x-web-markdown, or text/plain.
   * @param $as_user
   *   An email address representing the user to create the pad as.
   * @param $body
   *   The body of the new pad to create.
   *
   * @return
   *   The object returned by the HackPad API containing a padId property.
   */
  public function padCreate($content_type, $as_user, $body) {
    $options = array(
      'headers' => array(
        'content-type' => $content_type,
      ),
      'method' => 'POST',
      'data' => $body,
      'query' => array(
        'asUser' => $as_user,
      ),
    );

    return $this->request_json("pad/create", $options);
  }

  /**
   * Set the contents of a pad.
   *
   * @param $pad_id
   *   The Pad ID to set the contents of.
   * @param $as_user
   *   An email address representing the user to set the pad contents as.
   * @param $content_type
   *   A MIME content type indicating the type of $body. One of text/html,
   *   text/x-web-markdown, or text/plain.
   * @param $body
   *   The new body of the pad.
   *
   * @return
   *   The response data from the request.
   */
  public function padContent($pad_id, $as_user, $content_type, $body) {
    $resource = "pad/$pad_id/content";

    $options = array(
      'headers' => array(
        'content-type' => $content_type,
      ),
      'method' => 'POST',
      'data' => $body,
      'query' => array(
        'asUser' => $as_user,
       ),
    );

    return $this->request_json($resource, $options);
  }

  /**
   * Revert the contents of a pad.
   *
   * @param $pad_id
   *   The ID of the pad to revert.
   * @param $revision_id
   *   The revision ID to revert to.
   * @param $as_user
   *   Parameter of the email address of the user to run the revert as.
   *
   * @return
   *   A JSON object indicating if the revert was successful.
   */
  public function padRevert($pad_id, $revision_id, $as_user) {
    $resource = "pad/$pad_id/revert-to/$revision_id";

    $options = array(
      'method' => 'POST',
      'query' => array(
        'asUser' => $as_user,
      ),
    );

    return $this->request($resource, $options)->data;
  }

  /**
   * Load the contents of a pad.
   *
   * @param $pad_id
   *   The ID of the pad to load.
   * @param $as_user
   *   Parameter of the email address of the user to run the revert as.
   * @param $format
   *   Optional parameter of the format ('html', 'md', 'native', 'text') of the
   *   pad to load. Defaults to 'html'.
   * @param $revision_id
   *   Optional parameter of the revision of the pad to load. Defaults to
   *   'latest'.
   *
   * @return
   *   An object with 'revision' and 'body' properties containing the data
   *   loaded from the pad.
   */
  public function padContentLoad($pad_id, $as_user, $format = 'html', $revision_id = 'latest') {
    $resource = "pad/$pad_id/content/$revision_id.$format";
    
    $options = array(
      'method' => 'GET',
      'query' => array(
        'asUser' => $as_user,
      ),
    );
    $response = $this->request($resource);

    $pad = new stdClass();
    $pad->revision_id = $response->headers['x-hackpad-revision'];
    $pad->body = $response->data;

    return $pad;
  }

  /**
   * Return all of the revisions of a given pad.
   *
   * @param $pad_id
   *   The ID of the pad to load.
   *
   * @return
   *   An array of revision objects.
   */
  public function padRevisions($pad_id) {
    $resource = "pad/$pad_id/revisions";
    return $this->request_json($resource);
  }

  /**
   * Return an array of all pads that can be accessed by the calling user.
   *
   * @return
   *   An array of all pads that can be accessed by the calling user.
   */
  public function padsAll() {
    $resource = "pads/all";
    return $this->request_json($resource);
  }

  /**
   * Search Hackpad pads.
   *
   * @param $query
   *   The text string to search for.
   * @param $as_user
   *   The user to search as.
   * @param $start
   *   Optional parameter of the index to start search results at, used for
   *   paging.
   * @param $limit
   *   Optional parameter of the number of results to return, defaulting to 10.
   *
   * @return
   *   An array of objects containing the pad title, ID, and matched snippet.
   */
  public function search($query, $as_user, $start = 0, $limit = 10) {
    $resource = "search";
    $params = array(
      'q' => $query,
      'asUser' => $as_user,
      'start' => $start,
      'limit' => $limit,
    );

    return $this->request_json($resource, array('query' => $params));
  }

  /**
   * Return an array of pad IDs that have been edited since a given time.
   *
   * @param $timestamp
   *   A Unix timestamp.
   *
   * @return
   *   An array of pad IDs, limited to 1000.
   */
  public function editedSince($timestamp) {
    return $this->request_json("edited-since/$timestamp");
  }

  /**
   * Revoke access to a pad for a user.
   *
   * @param $pad_id
   *   The ID of the pad to revoke access for.
   * @param $user
   *   The email addres sof the user to revoke access for.
   * @param $as_user
   *   The user who performs the revokation.
   *
   * @return
   *   A JSON object indicating if the request was successful.
   */
  public function revokeAccess($pad_id, $user, $as_user) {
    return $this->request_json("pad/$pad_id/revoke-access/$user?$as_user");
  }

  /**
   * Set custom user settings on a Hackpad user.
   *
   * @param $user
   *   The user's email address.
   * @param $settings
   *   An array of settings, keyed by the setting name.
   *
   * @return
   *   A JSON success object.
   */
  public function userSettings($user, $settings) {
    return $this->request_json("user/$user/settings", array('query' => $settings, 'method' => 'POST'));
  }

  /**
   * Enable or disable Hackpad notifications for a user.
   *
   * @param $user
   *   The user's email address to disable notifications for.
   *
   * @param $enabled
   *   Boolean of the desired setting for notifications.
   *
   * @param
   *  A JSON success object.
   */
  public function userNotifications($user, $enabled) {
    if ($enabled) {
      $settings['send-email'] = 'true';
    }
    else {
      $settings['send-email'] = 'false';
    }

    return $this->userSettings($user, $settings);
  }

  /**
   * Return a URL suitable for passing as the url parameter to render_url() in
   * the JavaScript API.
   *
   * @param $pad_id
   *   The ID of the pad to embed.
   * @param $name
   *   The name of the user to show as connecting to the pad.
   * @param $mail
   *   The email address of the user connecting to the pad.
   *
   * @return
   *   A signed URL.
   */
  public function padRenderUrl($pad_id, $name, $mail) {
    $params = array(
      'padId' => $pad_id,
      'name' => $name,
      'email' => $mail,
    );
    $subdomain = variable_get('hackpad_subdomain', '');
    $url = 'https://'.$subdomain.'.hackpad.com/ep/api/embed-pad';

    return url($url, array('query' => $this->signRequest($url, 'GET', $params)));
  }
}

/**
 * Exception thrown if a HTTP request to HackPad fails.
 */
class HackpadRequestException extends Exception {}

